package FluxY_Lib::MS::DataClass_CW;

################################################################################
# This module provides support for cell wall compounds.
#
# This class inherits basic functionality from the DataClass parent class,
# initializing it with the cell wall compounds, the atom list and compound name.
# It implements the required PrintMassTables(). Two additional helper classes, 
#GenerateMassList() and GetMassList() are also implemented.
#
################################################################################

use strict;
use warnings;

use FluxY_Lib::Chemistry::atomInfo; # User Lib - Atomic Masses.
use FluxY_Lib::Math::Math_Utils;    # User Lib - Simple math functions.

use FluxY_Lib::MS::DataClass;       # User Lib - The common parent class.

our @ISA = qw(FluxY_Lib::MS::DataClass);    # Inherits from DataClass


# List of the atoms found in cell wall compounds and subsequently considerred
# for natural abundance. 
my @atomList = ('C','H','O','N','P');
my @atomMass = ();

# Each of the following atom counts is an array of the number of each corresponding
# atom relative to the order specified in the atomList, i.e. C, H, O, N and P.

# Hash of compound data, (first two array elements are required).
# 1. The compound names (cell wall), in lower case (for standard comparison).
# 2. The atom count for each cell wall compound.
my %compounds = (
   adpglc   => ["adp-glucose ",              [16,25,15,5,2]],
   dhap     => ["Dihydroxyacetone phosphate",[3,7,6,0,1]],
   e4p      => ["erythrose 4-phosphate",     [4,9,7,0,1]],
   fru6p    => ["fructose 6-phosphate",      [6,13,9,0,1]],
   frubp    => ["fructose 1,6-bisphosphate", [6,14,12,0,2]],
   glc1p    => ["glucose 1-phosphate",       [6,13,9,0,1]],
   glc6p    => ["glucose 6-phosphate",       [6,13,9,0,1]],
   gal1p    => ["galactose 1-phosphate",     [6,13,9,0,1]],
   gdpfuc   => ["gdp-fucose",                [16,25,15,5,2]],
   gdpglc   => ["gdp-glucose",               [16,25,16,5,2]],
   gdpman   => ["gdp-mannose",               [16,25,16,5,2]],
   gdphex   => ["gdp-hexose",                [16,25,16,5,2]],
   gmp      => ["guanosine monophosphate",   [10,14,8,5,1]],
   hex1p    => ["hexose 1-phosphate",        [6,13,9,0,1]],
   man1p    => ["mannose 1-phosphate",       [6,13,9,0,1]],
   man6p    => ["mannose 6-phosphate",       [6,13,9,0,1]],
   pg2_3    => ["2/3-phosphoglycerate",      [3,7,7,0,1]],
   pga      => ["3-phosphoglycerate",        [3,7,7,0,1]],
   pg6      => ["6-phosphoglucate",          [6,13,10,0,1]],
   r5p      => ["ribose 5-phosphate",        [5,11,8,0,1]],
   ribup    => ["ribose 5-phosphate",        [5,11,8,0,1]],
   ru5p     => ["Ribulose-5-phosphate",      [5,11,8,0,1]],
   rubp     => ["ribose-1,5-bisphosphate",   [5,12,11,0,2]],
   s7p      => ["sedoheptulose 7-phosphate", [7,15,10,0,1]],
	sbp      => ["sedoheptulose-1,7-bisphosphate", [7,16,13,0,2]],
   sucp     => ["sucrose 6-phosphate",       [12,23,14,0,1]],
   udpapi   => ["udp-apiose",                [14,22,16,2,2]],
   udpara   => ["udp-arabinose",             [14,22,16,2,2]],
   udpgal   => ["udp-galactose",             [15,24,17,2,2]],
   udpgaln  => ["udp-acetylgalactosamine",   [17,27,17,3,2]],
   udpgalte => ["udp-galacturonate",         [15,22,18,2,2]],
   udpglc   => ["udp-glucose",               [15,24,17,2,2]],
   udpglcn  => ["udp-acetylglucosamine",     [17,27,17,3,2]],
   udpglcte => ["udp-glucuronate",           [15,22,18,2,2]],
#   udprham  => ["udp-rhamnose",              [15,24,16,2,2]],
   udprha   => ["udp-rhamnose",              [15,24,16,2,2]],
   udpxyl   => ["udp-xylose",                [14,22,16,2,2]],
   ump      => ["uridine monophosphate",     [9,13, 9,2,1]],
   x5p      => ["xylulose 5-phosphate",      [5,11,8,0,1]],
   );
   
# A 2D Hash of compound masses, indexed by compunds key and frags keys.
# Each element is the truncated mass value for the given compound/fragment.
my %massList = ();


################################################################################
# Class constructor
#
# Used to instantiate an object for this class.
#
################################################################################
sub new {
   my ($class) = @_;
   
   # Initialize the atom mass vector.
   @atomMass = &GetElementMasses(@atomList);

   # Initialize the compound mass list.
   GenerateMassList();

   my $self = $class->SUPER::new(\%compounds, \@atomList, "Cell Wall (CW)");

   bless $self, $class;

   return $self;
}


######################################
#                                    #
#   Implemented DataClass methods.   #
#                                    #
######################################

################################################################################
# Subroutine: PrintMassTables
#
# Prints a table containing the atom count / mass for each compound.
#
################################################################################
sub PrintMassTables{
   my ($self) = @_;

   my $outFile = "CW_Data_Table.txt";

   open (FILE, ">" . $outFile) or FileSystemError(900, $outFile, $!);
   
   print FILE "Cell Wall Data Table:\n\n\t";

   print FILE "\t" . join("\t", @atomList) . "\tMass\tC-Backbone\n";

   foreach my $compound (sort keys %compounds) {
	   my $numCarbons = $self->GetNumCarbons($compound);
	
      print FILE $compound . "\t\t" .  join("\t", @{${$compounds{$compound}}[1]}) . "\t" . $massList{$compound}{'M-0'} . "\t1-" . $numCarbons . "\n";
   }
   
   close(FILE)                 or FileSystemError(901, $outFile, $!);
}


#######################
#                     #
#   Public methods.   #
#                     #
#######################

################################################################################
# Subroutine: GenerateMassList
#
# Generates a list of mass values in a hash (indexed by compound key) of hashes
# (indexed by fragment key).
#
################################################################################
sub GenerateMassList {
   foreach my $compound (sort keys %compounds) {
   
      my %list = ();
      
      my @totAtoms   = @{${$compounds{$compound}}[1]};
      
      my $mass = 0;
      for(my $i=0; $i<@totAtoms; $i++) {
         $mass += ($totAtoms[$i] * $atomMass[$i]);
      }
      
      $list{'M-0'} = &Trunc($mass);
      
      $massList{$compound} = \%list;
   }
}


################################################################################
# Subroutine: GetMassList
#
# Return the massList hash.
#
################################################################################
sub GetMassList {
   return %massList;
}

1;